/*
  pico bme280-log.c
  Use the bme280 or bmp280 (no humidity)
  Log the data in Flash
  Jed Margolin 7/8/2024
  board: rpi-pico-v6
  uart is 115,200 Baud, 1-start, 8 data, 1 stop, no parity
*/

#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "hardware/i2c.h"
#include "hardware/adc.h"
#include "hardware/spi.h"
#include "hardware/uart.h"
#include "hardware/structs/i2c.h"
#include "pico/binary_info.h"
#include "main.h"

#define ADC 0      // 1 = ADC Preamp is loaded (but not used here), 0 = ADC Preamp is not loaded
//================================================================
bool repeating_timer_callback(struct repeating_timer *t) 
{
   if(mode)    // 4 ms interrupts for BME-280 Flash Log
   {
      irq_count--;
      if(irq_count == 0)
      {
         irq_count = 250;   // 250 = 1 second
         secs_flag = 1;      // seconds flag
      }

   }
   else     // 4 ms interrupts for setting the ds3231 clock
   {
      irq_flag = 1;

      // read switches
      do_switch(PB1, &switch1);   // PB1, switch structure  INC
      do_switch(PB2, &switch2);   // PB2, switch structure  DEC
      do_switch(PB3, &switch3);   // PB3, switch structure  MODE
   
      irq_count--;
      if(irq_count == 0)  { irq_count = 250; }  // 250 = 1 second
   }   // end of mode

   return (true);
}
//===============================================================
int main () 
{
   nflag = 0;  // for flashing the LED

   lcd_init();

// Hardware on the board
   gpio_init(LED1);      // Green
   gpio_set_dir(LED1, GPIO_OUT);
   gpio_put(LED1,0);

   gpio_init(LED2);      // Red
   gpio_set_dir(LED2, GPIO_OUT);
   gpio_put(LED2,0);

   gpio_init(PB1);       // Pushbutton 1
   gpio_set_dir(PB1, GPIO_IN);
   gpio_pull_up(PB1);

   gpio_init(PB2);       // Pushbutton 2
   gpio_set_dir(PB2, GPIO_IN);
   gpio_pull_up(PB2);

   gpio_init(PB3);       // Pushbutton 3
   gpio_set_dir(PB3, GPIO_IN);
   gpio_pull_up(PB3);

   gpio_init(SW);        // Slide Switch
   gpio_set_dir(SW, GPIO_IN);
   gpio_pull_up(SW);

   gpio_init(CHANNEL);   // Channel (UART RX)
   gpio_set_dir(CHANNEL, GPIO_IN);
   gpio_pull_up(CHANNEL);
//-----------------------------------
// For the bme-280 and ds3231 RTC
// This uses I2C0 on the default SDA and SCL pins (4, 5)
   i2c_init(i2c_default, 300 * 1000);   // 300 KHz (400 KHz nax)
   gpio_set_function(PICO_DEFAULT_I2C_SDA_PIN, GPIO_FUNC_I2C);
   gpio_set_function(PICO_DEFAULT_I2C_SCL_PIN, GPIO_FUNC_I2C);
   gpio_pull_up(PICO_DEFAULT_I2C_SDA_PIN);
   gpio_pull_up(PICO_DEFAULT_I2C_SCL_PIN);
//    bi_decl(bi_2pins_with_func(PICO_DEFAULT_I2C_SDA_PIN, PICO_DEFAULT_I2C_SCL_PIN, GPIO_FUNC_I2C));
//-----------------------------------
// For the nrf905 RF module (not used here yet)
// Enable SPI 0 at 2 MHz and connect to GPIOs
   spi_init(spi_default, 2000 * 1000);
   gpio_set_function(PICO_DEFAULT_SPI_RX_PIN, GPIO_FUNC_SPI);
   gpio_set_function(PICO_DEFAULT_SPI_SCK_PIN, GPIO_FUNC_SPI);
   gpio_set_function(PICO_DEFAULT_SPI_TX_PIN, GPIO_FUNC_SPI);

   gpio_init(TXEN);    
   gpio_set_dir(TXEN, GPIO_OUT);
   gpio_put(TXEN,0);   // nrf905 Transmit Enable = High

   gpio_init(TRX_CE);    
   gpio_set_dir(TRX_CE, GPIO_OUT);
   gpio_put(TRX_CE,0);   // nrf905 Transmit Pulse = High

   gpio_init(DR);    
   gpio_set_dir(DR, GPIO_IN);
//   gpio_pull_up(DR);   // default is pull down

   gpio_init(AM);   // address match    
   gpio_set_dir(AM, GPIO_IN);
//   gpio_pull_up(AM);  // default is pull down

   gpio_init(CSN905);   // nrf905 CSN
   gpio_set_dir(CSN905, GPIO_OUT);
   gpio_put(CSN905,1);   

// RX mode
   gpio_put(TXEN,0);
   gpio_put(TRX_CE,1);

// Make the SPI pins available to picotool
//   bi_decl(bi_4pins_with_func(PICO_DEFAULT_SPI_RX_PIN, PICO_DEFAULT_SPI_TX_PIN, PICO_DEFAULT_SPI_SCK_PIN, PICO_DEFAULT_SPI_CSN_PIN, GPIO_FUNC_SPI));

// CLK idle state = low, data sampled on rising edge and shifted on falling edge
// Does not support LSB first; fortunately the nrf905 is MSB first  
   spi_set_format(spi_default, 8, 0, 0, SPI_MSB_FIRST);  

   nrf905_init();
//-------------------
// Set up the UART with the required speed.
   uart_init(UART_ID, BAUD_RATE);

// Set the TX and RX pins by using the function select on the GPIO
   gpio_set_function(UART_TX_PIN, GPIO_FUNC_UART);
   gpio_set_function(UART_RX_PIN, GPIO_FUNC_UART);

// Send the title message and version out through uart
   strcpy(line, version);
   strcat(line, "\n");
   uart_puts(UART_ID, line);   // send it out

   sleep_ms(1000);
//---------------------------------------------
#if ADC
// ADC Preamp is loaded but not used)
   adc_init();
   adc_gpio_init(26);    // Make sure GPIO is high-impedance, no pullups etc
   adc_select_input(0);  // Select ADC input 0 (GPIO-26, pin 31)
#else
// ADC Preamp is not loaded, make it a digital input
   gpio_init(ADC0_PIN);
   gpio_set_dir(ADC0_PIN, GPIO_IN);
   gpio_pull_up(ADC0_PIN);
#endif
//-------------------
   mode = gpio_get(SW);
   irq_flag = 0;
   irq_count = 250;
   secs_flag = 0;
   led_flag = 0;

   domsg(&dmsg1);   // "Jed Margolin's"
   domsg(&dmsg2);   // "BME-280 test"

   blink_led(500, 2);
 
   SetCmdMode();      // set for commands
   lcd_byte(0x01);    // clear screen
   domsg(&dmsg3);     // "bme-280 test"

   sleep_ms(2000); 
   SetCmdMode();      // set for commands

   i2c_outbuffer[0] = 0xD0;  // register address
   i2c_inbuffer[0] = 0x00;   
//---------------------------
// Get Flash memory ready
   find_first_blank_page();  // Find the first blank Page_Index (and Sector_index)

// If the SDATE page is blank, write today as the Start Date, otherwise show the Start Date
   do_start_date(0);  // Show Start Date

//LED_break();
//=====================================
   if(mode)   // bme-280
   {
      // Display the BME device id
      i2c_outbuffer[0] = 0xD0;  // register address
      i2c_write_blocking(i2c_default, BME_ID, i2c_outbuffer, 1, true);
      i2c_read_blocking(i2c_default, BME_ID, i2c_inbuffer, 1, false);
      device_id = i2c_inbuffer[0];
      
      SetCmdMode();      // set for commands
      lcd_byte(0x01);    // clear screen

      domsg(&dmsg4);   // "Device ID"
      lcd_wr_hex(0x8A,device_id);
      device_id = i2c_inbuffer[0];

      if(device_id == 0x60)   { domsg(&dmsg5); }   // "BME-280"
      if(device_id == 0x58)   { domsg(&dmsg6); }   // "BMP-280"
      if( (device_id != 0x60) && (device_id != 0x58) )  { domsg (&dmsg7); }  // Unknown

   // Create a repeating timer that calls repeating_timer_callback
      // If the delay is negative the next callback will be starting from the beginning of the callback
      //    Positive starts from the end of the callback
      add_repeating_timer_ms(-4,repeating_timer_callback, NULL, &timer);  // 4 ms for bme-280

      do_bme280();   // doesn't come back 
   }
   else
   {
      // Create a repeating timer that calls repeating_timer_callback
      // If the delay is negative the next callback will be starting from the beginning of the callback
      //    Positive starts from the end of the callback
      add_repeating_timer_ms(-4,repeating_timer_callback, NULL, &timer);  // 4 ms for clock

      do_clock();   // Display and set the ds3231; doesn't come back
   }
} // end main
//======================================================================
//======================================================================
void nrf905_init()
{
//   unsigned char n, lcd_pos;

// Writing the Configuration and TXADR address
   if( gpio_get(CHANNEL) )  
   {
      gpio_put(CSN905,0);
      spi_write_blocking(spi_default, WC1, 11);     // write Configuration  434.0 MHz
      gpio_put(CSN905,1);

      sleep_us(1000);

      gpio_put(CSN905,0);
      spi_write_blocking(spi_default, TXADR1, 5);
      gpio_put(CSN905,1);
   }
   else 
   {
      gpio_put(CSN905,0);
      spi_write_blocking(spi_default, WC2, 11);   // write Configuration  433.6 MHz  Test
      gpio_put(CSN905,1);
     
      sleep_us(1000);

      gpio_put(CSN905,0);
      spi_write_blocking(spi_default, TXADR2, 5);
      gpio_put(CSN905,1);
   }

   sleep_ms(100);

// Reading the Configuration
   spi_out_buffer[0] = 0x10;   // command for reading Configuration

   gpio_put(CSN905,0);   
   spi_write_read_blocking(spi_default, spi_out_buffer, spi_in_buffer, 11);
   gpio_put(CSN905,1);   

   SetCmdMode();    // set for commands
   lcd_byte(0x01);  // clear screen
}
//=================================================
// just toggle the LEDs
void do_led_break()
{
   while(1)
   {
      gpio_put(LED1,1);
      sleep_ms(1000);
 
      gpio_put(LED1,0);
      sleep_ms(1000);
   }
}
//================================================
void blink_led(unsigned int time, unsigned int times)
{
    unsigned int n;

    for(n=0; n<times; n++)
    { 
       gpio_put(LED1,1);
       gpio_put(LED2,0);
       sleep_ms(time);
       gpio_put(LED1,0);
       gpio_put(LED2,1);
       sleep_ms(time);
    }
    gpio_put(LED2,0);
}
//=============================================
